@@ -57,11 +57,13 @@ require_once("backend/combined/config.php");
 require_once("backend/combined/importer.php");
 require_once("backend/combined/exporter.php");
 
-class BackendCombined extends Backend {
+class BackendCombined extends Backend implements ISearchProvider {
     public $config;
     public $backends;
     private $activeBackend;
     private $activeBackendID;
+    private $numberChangesSink;
+    private $logon_done = false;
 
     /**
      * Constructor of the combined backend
@@ -72,12 +74,12 @@ class BackendCombined extends Backend {
         parent::Backend();
         $this->config = BackendCombinedConfig::GetBackendCombinedConfig();
 
-        foreach ($this->config['backends'] as $i => $b){
-            // load and instatiate backend
-            ZPush::IncludeBackend($b['name']);
-            $this->backends[$i] = new $b['name']();
+        $backend_values = array_unique(array_values($this->config['folderbackend']));
+        foreach ($backend_values as $i) {
+            ZPush::IncludeBackend($this->config['backends'][$i]['name']);
+            $this->backends[$i] = new $this->config['backends'][$i]['name']();
         }
-        ZLog::Write(LOGLEVEL_INFO, sprintf("Combined %d backends loaded.", count($this->backends)));
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("Combined %d backends loaded.", count($this->backends)));
     }
 
     /**
@@ -116,7 +118,9 @@ class BackendCombined extends Backend {
                 return false;
             }
         }
-        ZLog::Write(LOGLEVEL_INFO, "Combined->Logon() success");
+
+        $this->logon_done = true;
+        ZLog::Write(LOGLEVEL_DEBUG, "Combined->Logon() success");
         return true;
     }
 
@@ -154,7 +158,7 @@ class BackendCombined extends Backend {
                 return false;
             }
         }
-        ZLog::Write(LOGLEVEL_INFO, "Combined->Setup() success");
+        ZLog::Write(LOGLEVEL_DEBUG, "Combined->Setup() success");
         return true;
     }
 
@@ -165,6 +169,10 @@ class BackendCombined extends Backend {
      * @return boolean
      */
     public function Logoff() {
+        // If no Logon in done, omit Logoff
+        if (!$this->logon_done)
+            return true;
+
         ZLog::Write(LOGLEVEL_DEBUG, "Combined->Logoff()");
         foreach ($this->backends as $i => $b){
             $this->backends[$i]->Logoff();
@@ -274,6 +282,10 @@ class BackendCombined extends Backend {
      */
     public function SendMail($sm) {
         ZLog::Write(LOGLEVEL_DEBUG, "Combined->SendMail()");
+        // Convert source folderid
+        if (isset($sm->source->folderid)) {
+            $sm->source->folderid = $this->GetBackendFolder($sm->source->folderid);
+        }
         foreach ($this->backends as $i => $b){
             if($this->backends[$i]->SendMail($sm) == true){
                 return true;
@@ -358,11 +370,119 @@ class BackendCombined extends Backend {
      * @return string       id of the created/updated calendar obj
      * @throws StatusException
      */
-    public function MeetingResponse($requestid, $folderid, $error) {
+    public function MeetingResponse($requestid, $folderid, $response) {
         $backend = $this->GetBackend($folderid);
         if($backend === false)
             return false;
-        return $backend->MeetingResponse($requestid, $this->GetBackendFolder($folderid), $error);
+        return $backend->MeetingResponse($requestid, $this->GetBackendFolder($folderid), $response);
+    }
+
+
+    /**
+     * Deletes all contents of the specified folder.
+     * This is generally used to empty the trash (wastebasked), but could also be used on any
+     * other folder.
+     *
+     * @param string        $folderid
+     * @param boolean       $includeSubfolders      (opt) also delete sub folders, default true
+     *
+     * @access public
+     * @return boolean
+     * @throws StatusException
+     */
+    public function EmptyFolder($folderid, $includeSubfolders = true) {
+        $backend = $this->GetBackend($folderid);
+        if($backend === false)
+            return false;
+        return $backend->EmptyFolder($this->GetBackendFolder($folderid), $includeSubfolders);
+    }
+
+
+    /**
+     * Indicates if the backend has a ChangesSink.
+     * A sink is an active notification mechanism which does not need polling.
+     *
+     * @access public
+     * @return boolean
+     */
+    public function HasChangesSink() {
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCombined->HasChangesSink()"));
+
+        $this->numberChangesSink = 0;
+
+        foreach ($this->backends as $i => $b) {
+            if ($this->backends[$i]->HasChangesSink()) {
+                $this->numberChangesSink++;
+            }
+        }
+
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCombined->HasChangesSink - Number ChangesSink found: %d", $this->numberChangesSink));
+
+        return true;
+    }
+
+    /**
+     * The folder should be considered by the sink.
+     * Folders which were not initialized should not result in a notification
+     * of IBacken->ChangesSink().
+     *
+     * @param string        $folderid
+     *
+     * @access public
+     * @return boolean      false if there is any problem with that folder
+     */
+     public function ChangesSinkInitialize($folderid) {
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCombined->ChangesSinkInitialize('%s')", $folderid));
+
+        $backend = $this->GetBackend($folderid);
+        if($backend === false) {
+            // if not backend is found we return true, we don't want this to cause an error
+            return true;
+        }
+
+        if ($backend->HasChangesSink()) {
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCombined->ChangesSinkInitialize('%s') is supported, initializing", $folderid));
+            return $backend->ChangesSinkInitialize($this->GetBackendFolder($folderid));
+        }
+            
+        // if the backend doesn't support ChangesSink, we also return true so we don't get an error
+        return true;
+     }
+
+    /**
+     * The actual ChangesSink.
+     * For max. the $timeout value this method should block and if no changes
+     * are available return an empty array.
+     * If changes are available a list of folderids is expected.
+     *
+     * @param int           $timeout        max. amount of seconds to block
+     *
+     * @access public
+     * @return array
+     */
+    public function ChangesSink($timeout = 30) {
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCombined->ChangesSink(%d)", $timeout));
+
+        $notifications = array();
+        if ($this->numberChangesSink == 0) {
+            ZLog::Write(LOGLEVEL_DEBUG, "BackendCombined doesn't include any Sinkable backends");
+        } else {
+            $time_each = $timeout / $this->numberChangesSink;
+            foreach ($this->backends as $i => $b) {
+                if ($this->backends[$i]->HasChangesSink()) {
+                    ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCombined->ChangesSink - Calling in '%s' with %d", get_class($b), $time_each));
+
+                    $notifications_backend = $this->backends[$i]->ChangesSink($time_each);
+                    //preppend backend delimiter
+                    for ($c = 0; $c < count($notifications_backend); $c++) {
+                        $notifications_backend[$c] = $i . $this->config['delimiter'] . $notifications_backend[$c];
+                    }
+                    $notifications = array_merge($notifications, $notifications_backend);
+                }
+            }
+        }
+
+        return $notifications;
     }
 
     /**
@@ -415,5 +535,159 @@ class BackendCombined extends Backend {
             return false;
         return substr($folderid,0,$pos);
     }
+
+    /**
+     * Returns the BackendCombined as it implements the ISearchProvider interface
+     * This could be overwritten by the global configuration
+     *
+     * @access public
+     * @return object       Implementation of ISearchProvider
+     */
+    public function GetSearchProvider() {
+        return $this;
+    }
+
+
+    /**
+     * Indicates which AS version is supported by the backend.
+     * Return the lowest version supported by the backends used.
+     *
+     * @access public
+     * @return string       AS version constant
+     */
+    public function GetSupportedASVersion() {
+        $version = ZPush::ASV_14;
+        foreach ($this->backends as $i => $b) {
+            $subversion = $this->backends[$i]->GetSupportedASVersion();
+            if ($subversion < $version) {
+                $version = $subversion;
+            }
+        }
+        return $version;
+    }
+
+
+    /*-----------------------------------------------------------------------------------------
+    -- ISearchProvider
+    ------------------------------------------------------------------------------------------*/
+    /**
+     * Indicates if a search type is supported by this SearchProvider
+     * It supports all the search types, searches are delegated.
+     *
+     * @param string        $searchtype
+     *
+     * @access public
+     * @return boolean
+     */
+    public function SupportsType($searchtype) {
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("Combined->SupportsType('%s')", $searchtype));
+        $i = $this->getSearchBackend($searchtype);
+
+        return $i !== false;
+    }
+
+
+    /**
+     * Queries the LDAP backend
+     *
+     * @param string        $searchquery        string to be searched for
+     * @param string        $searchrange        specified searchrange
+     *
+     * @access public
+     * @return array        search results
+     */
+    public function GetGALSearchResults($searchquery, $searchrange) {
+        ZLog::Write(LOGLEVEL_DEBUG, "Combined->GetGALSearchResults()");
+        $i = $this->getSearchBackend(ISearchProvider::SEARCH_GAL);
+
+        $result = false;
+        if ($i !== false) {
+            $result = $this->backends[$i]->GetGALSearchResults($searchquery, $searchrange);
+        }
+
+        return $result;
+    }
+
+
+    /**
+     * Searches for the emails on the server
+     *
+     * @param ContentParameter $cpo
+     *
+     * @return array
+     */
+    public function GetMailboxSearchResults($cpo) {
+        ZLog::Write(LOGLEVEL_DEBUG, "Combined->GetMailboxSearchResults()");
+        $i = $this->getSearchBackend(ISearchProvider::SEARCH_MAILBOX);
+
+        $result = false;
+        if ($i !== false) {
+            //Convert $cpo GetSearchFolderid
+            $cpo->SetSearchFolderid($this->GetBackendFolder($cpo->GetSearchFolderid()));
+            $result = $this->backends[$i]->GetMailboxSearchResults($cpo, $i . $this->config['delimiter']);
+        }
+
+        return $result;
+    }
+
+
+    /**
+    * Terminates a search for a given PID
+    *
+    * @param int $pid
+    *
+    * @return boolean
+    */
+    public function TerminateSearch($pid) {
+        ZLog::Write(LOGLEVEL_DEBUG, "Combined->TerminateSearch()");
+        foreach ($this->backends as $i => $b) {
+            if ($this->backends[$i] instanceof ISearchProvider) {
+                $this->backends[$i]->TerminateSearch($pid);
+            }
+        }
+
+        return true;
+    }
+
+
+    /**
+     * Disconnects backends
+     *
+     * @access public
+     * @return boolean
+     */
+    public function Disconnect() {
+        ZLog::Write(LOGLEVEL_DEBUG, "Combined->Disconnect()");
+        foreach ($this->backends as $i => $b) {
+            if ($this->backends[$i] instanceof ISearchProvider) {
+                $this->backends[$i]->Disconnect();
+            }
+        }
+
+        return true;
+    }
+
+
+    /**
+     * Returns the first backend that support a search type
+     *
+     * @param string    $searchtype
+     *
+     * @access private
+     * @return string
+     */
+    private function getSearchBackend($searchtype) {
+        foreach ($this->backends as $i => $b) {
+            if ($this->backends[$i] instanceof ISearchProvider) {
+                if ($this->backends[$i]->SupportsType($searchtype)) {
+                    return $i;
+                }
+            }
+        }
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("Combined->getSearchBackend('%s') No support found!", $searchtype));
+
+        return false;
+    }
+
 }
 ?>
\ No newline at end of file
