@@ -205,6 +205,7 @@ class MAPIMapping {
             "representingentryid"   => PR_SENT_REPRESENTING_ENTRYID,
             "sourcekey"             => PR_SOURCE_KEY,
             "entryid"               => PR_ENTRYID,
+            "parentsourcekey"       => PR_PARENT_SOURCE_KEY,
             "body"                  => PR_BODY,
             "rtfcompressed"         => PR_RTF_COMPRESSED,
             "html"                  => PR_HTML,
@@ -378,6 +379,7 @@ class MAPIMapping {
             "startdate"             => "PT_SYSTIME:PSETID_Task:0x8104",
             "subject"               => PR_SUBJECT,
             "rtf"                   => PR_RTF_COMPRESSED,
+            "html"                  => PR_HTML,
             );
     }
 
@@ -518,6 +520,8 @@ class MAPIMapping {
                 "attachnum"             => PR_ATTACH_NUM,
                 "attachdatabin"         => PR_ATTACH_DATA_BIN,
                 "internetcpid"          => PR_INTERNET_CPID,
+                "rtf"                   => PR_RTF_COMPRESSED,
+                "rtfinsync"             => PR_RTF_IN_SYNC,
         );
     }
 }
