@@ -164,7 +164,15 @@ class BackendZarafa implements IBackend, ISearchProvider {
         try {
             // check if notifications are available in php-mapi
             if(function_exists('mapi_feature') && mapi_feature('LOGONFLAGS')) {
-                $this->session = @mapi_logon_zarafa($user, $pass, MAPI_SERVER, null, null, 0);
+                // send Z-Push version and user agent to ZCP - ZP-589
+                if (Utils::CheckMapiExtVersion('7.2.0')) {
+                    $zpush_version = 'Z-Push_' . @constant('ZPUSH_VERSION');
+                    $user_agent = (Request::GetDeviceID()) ? ZPush::GetDeviceManager()->GetUserAgent() : "unknown";
+                    $this->session = @mapi_logon_zarafa($user, $pass, MAPI_SERVER, null, null, 0, $zpush_version, $user_agent);
+                }
+                else {
+                    $this->session = @mapi_logon_zarafa($user, $pass, MAPI_SERVER, null, null, 0);
+                }
                 $this->notifications = true;
             }
             // old fashioned session
@@ -453,18 +461,14 @@ class BackendZarafa implements IBackend, ISearchProvider {
         // @see http://jira.zarafa.com/browse/ZP-68
         $meetingRequestProps = MAPIMapping::GetMeetingRequestProperties();
         $meetingRequestProps = getPropIdsFromStrings($this->store, $meetingRequestProps);
-        $props = mapi_getprops($mapimessage, array(PR_MESSAGE_CLASS, $meetingRequestProps["goidtag"], $sendMailProps["internetcpid"]));
+        $props = mapi_getprops($mapimessage, array(PR_MESSAGE_CLASS, $meetingRequestProps["goidtag"], $sendMailProps["internetcpid"], $sendMailProps["body"], $sendMailProps["html"], $sendMailProps["rtf"], $sendMailProps["rtfinsync"]));
 
-        // Convert sent message's body to UTF-8.
-        // @see http://jira.zarafa.com/browse/ZP-505
-        if (isset($props[$sendMailProps["internetcpid"]]) && $props[$sendMailProps["internetcpid"]] != INTERNET_CPID_UTF8) {
-            ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sent email cpid is not unicode (%d). Set it to unicode and convert email body.", $props[$sendMailProps["internetcpid"]]));
+        // Convert sent message's body to UTF-8 if it was a HTML message.
+        // @see http://jira.zarafa.com/browse/ZP-505 and http://jira.zarafa.com/browse/ZP-555
+        if (isset($props[$sendMailProps["internetcpid"]]) && $props[$sendMailProps["internetcpid"]] != INTERNET_CPID_UTF8 && MAPIUtils::GetNativeBodyType($props) == SYNC_BODYPREFERENCE_HTML) {
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sent email cpid is not unicode (%d). Set it to unicode and convert email html body.", $props[$sendMailProps["internetcpid"]]));
             $mapiprops[$sendMailProps["internetcpid"]] = INTERNET_CPID_UTF8;
 
-            $body = MAPIUtils::readPropStream($mapimessage, PR_BODY);
-            $body = Utils::ConvertCodepageStringToUtf8($props[$sendMailProps["internetcpid"]], $body);
-            $mapiprops[$sendMailProps["body"]] = $body;
-
             $bodyHtml = MAPIUtils::readPropStream($mapimessage, PR_HTML);
             $bodyHtml = Utils::ConvertCodepageStringToUtf8($props[$sendMailProps["internetcpid"]], $bodyHtml);
             $mapiprops[$sendMailProps["html"]] = $bodyHtml;
@@ -506,10 +510,25 @@ class BackendZarafa implements IBackend, ISearchProvider {
             if(!isset($fwmessage) || !$fwmessage)
                 throw new StatusException(sprintf("ZarafaBackend->SendMail(): Could not open message id '%s' in folder id '%s' to be replied/forwarded: 0x%X", $sm->source->itemid, $sm->source->folderid, mapi_last_hresult()), SYNC_COMMONSTATUS_ITEMNOTFOUND);
 
-            //update icon when forwarding or replying message
-            if ($sm->forwardflag) mapi_setprops($fwmessage, array(PR_ICON_INDEX=>262));
-            elseif ($sm->replyflag) mapi_setprops($fwmessage, array(PR_ICON_INDEX=>261));
-            mapi_savechanges($fwmessage);
+            // update icon and last_verb when forwarding or replying message
+            // reply-all (verb 103) is not supported, as we cannot really detect this case
+            if ($sm->forwardflag) {
+                $updateProps = array(
+                    PR_ICON_INDEX           => 262,
+                    PR_LAST_VERB_EXECUTED   => 104,
+                );
+            }
+            elseif ($sm->replyflag) {
+                $updateProps = array(
+                    PR_ICON_INDEX           => 261,
+                    PR_LAST_VERB_EXECUTED   => 102,
+                );
+            }
+            if (isset($updateProps)) {
+                $updateProps[PR_LAST_VERB_EXECUTION_TIME] = time();
+                mapi_setprops($fwmessage, $updateProps);
+                mapi_savechanges($fwmessage);
+            }
 
             // only attach the original message if the mobile does not send it itself
             if (!isset($sm->replacemime)) {
@@ -642,13 +661,17 @@ class BackendZarafa implements IBackend, ISearchProvider {
         if(!strpos($attname, ":"))
             throw new StatusException(sprintf("ZarafaBackend->GetAttachmentData('%s'): Error, attachment requested for non-existing item", $attname), SYNC_ITEMOPERATIONSSTATUS_INVALIDATT);
 
-        list($id, $attachnum) = explode(":", $attname);
+        list($id, $attachnum, $parentEntryid) = explode(":", $attname);
+        if (isset($parentEntryid)) {
+            $this->Setup(ZPush::GetAdditionalSyncFolderStore($parentEntryid));
+        }
 
         $entryid = hex2bin($id);
         $message = mapi_msgstore_openentry($this->store, $entryid);
         if(!$message)
             throw new StatusException(sprintf("ZarafaBackend->GetAttachmentData('%s'): Error, unable to open item for attachment data for id '%s' with: 0x%X", $attname, $id, mapi_last_hresult()), SYNC_ITEMOPERATIONSSTATUS_INVALIDATT);
 
+        MAPIUtils::ParseSmime($this->session, $this->defaultstore, $this->getAddressbook(), $message);
         $attach = mapi_message_openattach($message, $attachnum);
         if(!$attach)
             throw new StatusException(sprintf("ZarafaBackend->GetAttachmentData('%s'): Error, unable to open attachment number '%s' with: 0x%X", $attname, $attachnum, mapi_last_hresult()), SYNC_ITEMOPERATIONSSTATUS_INVALIDATT);
@@ -968,9 +991,9 @@ class BackendZarafa implements IBackend, ISearchProvider {
      * @throws StatusException
      */
     public function GetGALSearchResults($searchquery, $searchrange){
-        // only return users from who the displayName or the username starts with $name
+        // only return users whose displayName or the username starts with $name
         //TODO: use PR_ANR for this restriction instead of PR_DISPLAY_NAME and PR_ACCOUNT
-        $addrbook = mapi_openaddressbook($this->session);
+        $addrbook = $this->getAddressbook();
         if ($addrbook)
             $ab_entryid = mapi_ab_getdefaultdir($addrbook);
         if ($ab_entryid)
@@ -1003,12 +1026,16 @@ class BackendZarafa implements IBackend, ISearchProvider {
         $querycnt = mapi_table_getrowcount($table);
         //do not return more results as requested in range
         $querylimit = (($rangeend + 1) < $querycnt) ? ($rangeend + 1) : $querycnt;
-        $items['range'] = ($querylimit > 0) ? $rangestart.'-'.($querylimit - 1) : '0-0';
-        $items['searchtotal'] = $querycnt;
+
         if ($querycnt > 0)
             $abentries = mapi_table_queryrows($table, array(PR_ACCOUNT, PR_DISPLAY_NAME, PR_SMTP_ADDRESS, PR_BUSINESS_TELEPHONE_NUMBER, PR_GIVEN_NAME, PR_SURNAME, PR_MOBILE_TELEPHONE_NUMBER, PR_HOME_TELEPHONE_NUMBER, PR_TITLE, PR_COMPANY_NAME, PR_OFFICE_LOCATION), $rangestart, $querylimit);
 
         for ($i = 0; $i < $querylimit; $i++) {
+            if (!isset($abentries[$i][PR_SMTP_ADDRESS])) {
+                ZLog::Write(LOGLEVEL_WARN, sprintf("The GAL entry '%s' does not have an email address and will be ignored.", w2u($abentries[$i][PR_DISPLAY_NAME])));
+                continue;
+            }
+
             $items[$i][SYNC_GAL_DISPLAYNAME] = w2u($abentries[$i][PR_DISPLAY_NAME]);
 
             if (strlen(trim($items[$i][SYNC_GAL_DISPLAYNAME])) == 0)
@@ -1047,6 +1074,9 @@ class BackendZarafa implements IBackend, ISearchProvider {
             if (isset($abentries[$i][PR_OFFICE_LOCATION]))
                 $items[$i][SYNC_GAL_OFFICE] = w2u($abentries[$i][PR_OFFICE_LOCATION]);
         }
+        $nrResults = count($items);
+        $items['range'] = ($nrResults > 0) ? $rangestart.'-'.($nrResults - 1) : '0-0';
+        $items['searchtotal'] = $nrResults;
         return $items;
     }
 
@@ -1117,6 +1147,11 @@ class BackendZarafa implements IBackend, ISearchProvider {
     */
     public function TerminateSearch($pid) {
         ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZarafaBackend->TerminateSearch(): terminating search for pid %d", $pid));
+        if (!isset($this->store) || $this->store === false) {
+            ZLog::Write(LOGLEVEL_WARN, sprintf("The store is not available. It is not possible to remove search folder with pid %d", $pid));
+            return false;
+        }
+
         $storeProps = mapi_getprops($this->store, array(PR_STORE_SUPPORT_MASK, PR_FINDER_ENTRYID));
         if (($storeProps[PR_STORE_SUPPORT_MASK] & STORE_SEARCH_OK) != STORE_SEARCH_OK) {
             ZLog::Write(LOGLEVEL_WARN, "Store doesn't support search folders. Public store doesn't have FINDER_ROOT folder");
