@@ -43,7 +43,13 @@
 * Consult LICENSE file for details
 ************************************************/
 
-ob_start(null, 1048576);
+
+if (version_compare(phpversion(), '5.4.0') < 0) {
+    ob_start(null, 1048576);
+}
+else {
+    ob_start(null, 1048576, PHP_OUTPUT_HANDLER_STDFLAGS);
+}
 
 // ignore user abortions because this can lead to weird errors - see ZP-239
 ignore_user_abort(true);
@@ -52,6 +58,7 @@ include_once('lib/exceptions/exceptions.php');
 include_once('lib/utils/utils.php');
 include_once('lib/utils/compat.php');
 include_once('lib/utils/timezoneutil.php');
+include_once('lib/utils/stringstreamwrapper.php');
 include_once('lib/core/zpushdefs.php');
 include_once('lib/core/stateobject.php');
 include_once('lib/core/interprocessdata.php');
@@ -63,7 +70,6 @@ include_once('lib/core/statemanager.php');
 include_once('lib/core/devicemanager.php');
 include_once('lib/core/zpush.php');
 include_once('lib/core/zlog.php');
-include_once('lib/core/paddingfilter.php');
 include_once('lib/interface/ibackend.php');
 include_once('lib/interface/ichanges.php');
 include_once('lib/interface/iexportchanges.php');
@@ -119,7 +125,8 @@ include_once('lib/default/searchprovider.php');
 include_once('lib/request/request.php');
 include_once('lib/request/requestprocessor.php');
 
-include_once('config.php');
+if (!defined('ZPUSH_CONFIG')) define('ZPUSH_CONFIG', 'config.php');
+include_once(ZPUSH_CONFIG);
 include_once('version.php');
 
 // Define apache_request_headers() in case we are using nginx or lighttpd
@@ -202,8 +209,7 @@ if (!function_exists("apache_request_headers")) {
         }
 
         RequestProcessor::Initialize();
-        if(!RequestProcessor::HandleRequest())
-            throw new WBXMLException(ZLog::GetWBXMLDebugInfo());
+        RequestProcessor::HandleRequest();
 
         // eventually the RequestProcessor wants to send other headers to the mobile
         foreach (RequestProcessor::GetSpecialHeaders() as $header)
@@ -229,7 +235,7 @@ if (!function_exists("apache_request_headers")) {
 
         // send vnd.ms-sync.wbxml content type header if there is no content
         // otherwise text/html content type is added which might break some devices
-        if ($len == 0)
+        if (!headers_sent() && $len == 0)
             header("Content-Type: application/vnd.ms-sync.wbxml");
 
         print $data;
@@ -281,7 +287,7 @@ if (!function_exists("apache_request_headers")) {
 
         // This could be a WBXML problem.. try to get the complete request
         else if ($ex instanceof WBXMLException) {
-            ZLog::Write(LOGLEVEL_FATAL, "Request could not be processed correctly due to a WBXMLException. Please report this.");
+            ZLog::Write(LOGLEVEL_FATAL, "Request could not be processed correctly due to a WBXMLException. Please report this including WBXML debug data logged. Be aware that the debug data could contain confidential information.");
         }
 
         // Try to output some kind of error information. This is only possible if
@@ -307,5 +313,5 @@ if (!function_exists("apache_request_headers")) {
         ZPush::GetDeviceManager()->Save();
 
     // end gracefully
-    ZLog::Write(LOGLEVEL_DEBUG, '-------- End');
+    ZLog::WriteEnd();
 ?>
