@@ -11,7 +11,7 @@
 *
 * Created   :   20.10.2011
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -178,6 +178,20 @@ class LoopDetection extends InterProcessData {
         return true;
     }
 
+    /**
+     * Marks the current process as a PUSH connection
+     *
+     * @access public
+     * @return boolean
+     */
+    public function ProcessLoopDetectionSetAsPush() {
+        // generate entry if not already there
+        self::GetProcessEntry();
+        self::$processentry['push'] = true;
+
+        return $this->updateProcessStack();
+    }
+
     /**
      * Indicates if a full Hierarchy Resync is necessary
      *
@@ -255,7 +269,7 @@ class LoopDetection extends InterProcessData {
         $stack = $this->getProcessStack();
         if (count($stack) > 1) {
             $se = $stack[0];
-            if (!isset($se['end']) && $se['cc'] != ZPush::COMMAND_PING) {
+            if (!isset($se['end']) && $se['cc'] != ZPush::COMMAND_PING && !isset($se['push']) ) {
                 // there is no end time
                 ZLog::Write(LOGLEVEL_ERROR, sprintf("LoopDetection->ProcessLoopDetectionPreviousConnectionFailed(): Command '%s' at %s with pid '%d' terminated unexpectedly or is still running.", Utils::GetCommandFromCode($se['cc']), Utils::GetFormattedTime($se['time']), $se['pid']));
                 ZLog::Write(LOGLEVEL_ERROR, "Please check your logs for this PID and errors like PHP-Fatals or Apache segmentation faults and report your results to the Z-Push dev team.");
@@ -501,6 +515,16 @@ class LoopDetection extends InterProcessData {
             $this->checkArrayStructure($loopdata, $folderid);
             $current = $loopdata[self::$devid][self::$user][$folderid];
 
+            if (!isset($current["uuid"])) {
+                $current["uuid"] = $uuid;
+            }
+            if (!isset($current["count"])) {
+                $current["count"] = $counter;
+            }
+            if (!isset($current["queued"])) {
+                $current["queued"] = 0;
+            }
+
             // update the usage flag
             $current["usage"] = $counter;
 
@@ -543,18 +567,28 @@ class LoopDetection extends InterProcessData {
             $current = $loopdata[self::$devid][self::$user][$folderid];
 
             if (!empty($current)) {
-                if ($current["uuid"] != $uuid) {
-                    ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, uuid changed");
+                if (!isset($current["uuid"]) || $current["uuid"] != $uuid) {
+                    ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, uuid changed or not set");
                     $obsolete = true;
                 }
-                ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->IsSyncStateObsolete(): check uuid counter: %d - last known counter: %d with %d queued objects", $counter, $current["count"], $current["queued"]));
+                else {
+                    ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->IsSyncStateObsolete(): check folderid: '%s' uuid '%s' counter: %d - last counter: %d with %d queued", 
+                            $folderid, $uuid, $counter, $current["count"], $current["queued"]));
 
-                if ($current["uuid"] == $uuid && ($current["count"] > $counter || ($current["count"] == $counter && $current["queued"] > 0) || (isset($current["usage"]) && $current["usage"] >= $counter))) {
-                    $usage = isset($current["usage"]) ? sprintf(" - counter %d already expired",$current["usage"]) : "";
-                    ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, counter already processed". $usage);
-                    $obsolete = true;
+                    if ($current["uuid"] == $uuid && (
+                            $current["count"] > $counter || 
+                            ($current["count"] == $counter && $current["queued"] > 0) || 
+                            (isset($current["usage"]) && $current["usage"] >= $counter)
+                          )) {
+                        $usage = isset($current["usage"]) ? sprintf(" - counter %d already expired",$current["usage"]) : "";
+                        ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, counter already processed". $usage);
+                        $obsolete = true;
+                    }
                 }
             }
+            else {
+                ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->IsSyncStateObsolete(): check folderid: '%s' uuid '%s' counter: %d - no data found: not obsolete", $folderid, $uuid, $counter));
+            }
         }
 
         return $obsolete;
@@ -592,6 +626,7 @@ class LoopDetection extends InterProcessData {
      * @return boolean      when returning true if a loop has been identified
      */
     public function Detect($folderid, $type, $uuid, $counter, $maxItems, $queuedMessages) {
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->Detect(): folderid:'%s' type:'%s' uuid:'%s' counter:'%s' max:'%s' queued:'%s'", $folderid, $type, $uuid, $counter, $maxItems, $queuedMessages));
         $this->broken_message_uuid = $uuid;
         $this->broken_message_counter = $counter;
 
@@ -619,8 +654,14 @@ class LoopDetection extends InterProcessData {
             if (empty($current))
                 $current = array("type" => $type, "uuid" => $uuid, "count" => $counter-1, "queued" => $queuedMessages);
 
+            // if data was created by SetSyncStateUsage(), we need to set the type
+            // TODO ZP-741: type is not used anywhere and could be removed from the loop data
+            if (isset($current['uuid']) && $current['uuid'] == $uuid && !isset($current['type'])) {
+                $current['type'] = $type;
+            }
+
             // old UUID in cache - the device requested a new state!!
-            else if (isset($current['type']) && $current['type'] == $type && isset($current['uuid']) && $current['uuid'] != $uuid ) {
+            if (isset($current['type']) && $current['type'] == $type && isset($current['uuid']) && $current['uuid'] != $uuid ) {
                 ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->Detect(): UUID changed for folder");
 
                 // some devices (iPhones) may request new UUIDs after broken items were sent several times
@@ -665,6 +706,7 @@ class LoopDetection extends InterProcessData {
                         // case 1.2.1
                         // broken item not identified yet
                         if (!isset($current['ignored']) && $counter < $current['maxCount']) {
+                            $current['loopcount'] = 1;
                             $loop = true; // continue in loop-resolving
                             ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->Detect(): case 1.2.1 detected");
                         }
