@@ -6,7 +6,7 @@
 *
 * Created   :   12.04.2011
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2015 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -109,11 +109,8 @@ class ZPush {
                     "BackendMaildir"
                 );
 
+    // Versions 1.0, 2.0, 2.1 and 2.5 are deprecated (ZP-604)
     static private $supportedASVersions = array(
-                    self::ASV_1,
-                    self::ASV_2,
-                    self::ASV_21,
-                    self::ASV_25,
                     self::ASV_12,
                     self::ASV_121,
                     self::ASV_14
@@ -488,7 +485,7 @@ class ZPush {
                 $loaded = false;
                 foreach (self::$autoloadBackendPreference as $autoloadBackend) {
                     ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPush::GetBackend(): trying autoload backend '%s'", $autoloadBackend));
-                    $loaded = self::IncludeBackend($autoloadBackend);
+                    $loaded = class_exists($autoloadBackend) || self::IncludeBackend($autoloadBackend);
                     if ($loaded) {
                         $ourBackend = $autoloadBackend;
                         break;
@@ -497,7 +494,7 @@ class ZPush {
                 if (!$ourBackend || !$loaded)
                     throw new FatalMisconfigurationException("No Backend provider can not be loaded. Check your installation and configuration!");
             }
-            else
+            elseif (!class_exists($ourBackend))
                 self::IncludeBackend($ourBackend);
 
             if (class_exists($ourBackend))
@@ -623,7 +620,8 @@ class ZPush {
         More information about Z-Push can be found at:<br>
         <a href="http://z-push.org/">Z-Push homepage</a><br>
         <a href="http://z-push.org/download">Z-Push download page</a><br>
-        <a href="http://jira.zarafa.com/browse/ZP">Z-Push Bugtracker and Roadmap</a><br>
+        <a href="https://jira.z-hub.io/browse/ZP">Z-Push Bugtracker</a><br>
+        <a href="https://wiki.z-hub.io/display/ZP">Z-Push Wiki</a> and <a href="https://wiki.z-hub.io/display/ZP/Roadmap">Roadmap</a><br>
         <br>
         All modifications to this sourcecode must be published and returned to the community.<br>
         Please see <a href="http://www.gnu.org/licenses/agpl-3.0.html">AGPLv3 License</a> for details.<br>
