@@ -126,10 +126,11 @@ class FileStateMachine implements IStateMachine {
         // Read current sync state
         $filename = $this->getFullFilePath($devid, $type, $key, $counter);
 
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->GetState() on file: '%s'", $filename));
-
         if(file_exists($filename)) {
-            return unserialize(file_get_contents($filename));
+            $contents = file_get_contents($filename);
+            $bytes = strlen($contents);
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->GetState() read '%d' bytes from file: '%s'", $bytes, $filename ));
+            return unserialize($contents);
         }
         // throw an exception on all other states, but not FAILSAVE as it's most of the times not there by default
         else if ($type !== IStateMachine::FAILSAVE)
@@ -331,8 +332,15 @@ class FileStateMachine implements IStateMachine {
      * @return int
      */
     public function GetStateVersion() {
-        if (file_exists($this->settingsfilename))
+        if (file_exists($this->settingsfilename)) {
             $settings = unserialize(file_get_contents($this->settingsfilename));
+            if (strtolower(gettype($settings) == "string") && strtolower($settings) == '2:1:{s:7:"version";s:1:"2";}') {
+                ZLog::Write(LOGLEVEL_INFO, "Broken state version file found. Attempt to autofix it. See https://jira.zarafa.com/browse/ZP-493 for more information.");
+                unlink($this->settingsfilename);
+                $this->SetStateVersion(IStateMachine::STATEVERSION_02);
+                $settings = array(self::VERSION => IStateMachine::STATEVERSION_02);
+            }
+        }
         else {
             $filecontents = @file_get_contents($this->userfilename);
             if ($filecontents)
@@ -356,9 +364,9 @@ class FileStateMachine implements IStateMachine {
      */
     public function SetStateVersion($version) {
         if (file_exists($this->settingsfilename))
-            $settings = file_get_contents($this->settingsfilename);
+            $settings = unserialize(file_get_contents($this->settingsfilename));
         else
-            array(self::VERSION => IStateMachine::STATEVERSION_01);
+            $settings = array(self::VERSION => IStateMachine::STATEVERSION_01);
 
         $settings[self::VERSION] = $version;
         ZLog::Write(LOGLEVEL_INFO, sprintf("FileStateMachine->SetStateVersion() saving supported state version, value '%d'", $version));
@@ -376,6 +384,7 @@ class FileStateMachine implements IStateMachine {
      * @return array(mixed)
      */
     public function GetAllStatesForDevice($devid) {
+        $types = array(IStateMachine::DEVICEDATA, IStateMachine::FOLDERDATA, IStateMachine::FAILSAVE, IStateMachine::HIERARCHY, IStateMachine::BACKENDSTORAGE);
         $out = array();
         $devdir = $this->getDirectoryForDevice($devid) . "/$devid-";
 
@@ -385,8 +394,9 @@ class FileStateMachine implements IStateMachine {
 
             $state = array('type' => false, 'counter' => false, 'uuid' => false);
 
-            if (isset($parts[0]) && $parts[0] == IStateMachine::DEVICEDATA)
-                $state['type'] = IStateMachine::DEVICEDATA;
+            // part 0 could be "devicedata" or another type in broken states
+            if (isset($parts[0]) && in_array($parts[0], $types))
+                $state['type'] = $parts[0];
 
             if (isset($parts[0]) && strlen($parts[0]) == 8 &&
                 isset($parts[1]) && strlen($parts[1]) == 4 &&
@@ -404,7 +414,7 @@ class FileStateMachine implements IStateMachine {
                 if (is_int($parts[5]))
                     $state['counter'] = $parts[5];
 
-                else if (in_array($parts[5], array(IStateMachine::FOLDERDATA, IStateMachine::FAILSAVE, IStateMachine::HIERARCHY, IStateMachine::BACKENDSTORAGE)))
+                else if (in_array($parts[5], $types))
                     $state['type'] = $parts[5];
             }
             if (isset($parts[6]) && is_numeric($parts[6]))
