@@ -113,8 +113,12 @@ class Request {
             self::$command = self::filterEvilInput($_GET["Cmd"], self::LETTERS_ONLY);
 
         // getUser is unfiltered, as everything is allowed.. even "/", "\" or ".."
-        if(isset($_GET["User"]))
+        if(isset($_GET["User"])) {
             self::$getUser = strtolower($_GET["User"]);
+            if(defined('USE_FULLEMAIL_FOR_LOGIN') && ! USE_FULLEMAIL_FOR_LOGIN) {
+                self::$getUser = Utils::GetLocalPartFromEmail(self::$getUser);
+            }
+        }
         if(isset($_GET["DeviceId"]))
             self::$devid = strtolower(self::filterEvilInput($_GET["DeviceId"], self::WORDCHAR_ONLY));
         if(isset($_GET["DeviceType"]))
@@ -140,8 +144,12 @@ class Request {
             if (!isset(self::$command) && isset($query['Command']))
                 self::$command = Utils::GetCommandFromCode($query['Command']);
 
-            if (!isset(self::$getUser) && isset($query[self::COMMANDPARAM_USER]))
+            if (!isset(self::$getUser) && isset($query[self::COMMANDPARAM_USER])) {
                 self::$getUser = strtolower($query[self::COMMANDPARAM_USER]);
+                if(defined('USE_FULLEMAIL_FOR_LOGIN') && ! USE_FULLEMAIL_FOR_LOGIN) {
+                    self::$getUser = Utils::GetLocalPartFromEmail(self::$getUser);
+                }
+            }
 
             if (!isset(self::$devid) && isset($query['DevID']))
                 self::$devid = strtolower(self::filterEvilInput($query['DevID'], self::WORDCHAR_ONLY));
@@ -172,8 +180,12 @@ class Request {
         }
 
         // in base64 encoded query string user is not necessarily set
-        if (!isset(self::$getUser) && isset($_SERVER['PHP_AUTH_USER']))
+        if (!isset(self::$getUser) && isset($_SERVER['PHP_AUTH_USER'])) {
             list(self::$getUser,) = Utils::SplitDomainUser(strtolower($_SERVER['PHP_AUTH_USER']));
+            if(defined('USE_FULLEMAIL_FOR_LOGIN') && ! USE_FULLEMAIL_FOR_LOGIN) {
+                self::$getUser = Utils::GetLocalPartFromEmail(self::$getUser);
+            }
+        }
     }
 
     /**
@@ -233,6 +245,9 @@ class Request {
             list(self::$authUser, self::$authDomain) = Utils::SplitDomainUser($_SERVER['PHP_AUTH_USER']);
             self::$authPassword = (isset($_SERVER['PHP_AUTH_PW']))?$_SERVER['PHP_AUTH_PW'] : "";
         }
+        if(defined('USE_FULLEMAIL_FOR_LOGIN') && ! USE_FULLEMAIL_FOR_LOGIN) {
+            self::$authUser = Utils::GetLocalPartFromEmail(self::$authUser);
+        }
         // authUser & authPassword are unfiltered!
         return (self::$authUser != "" && self::$authPassword != "");
     }
@@ -566,6 +581,32 @@ class Request {
         return (isset(self::$headers["content-length"]))? (int) self::$headers["content-length"] : 0;
     }
 
+    /**
+     * Returns the amount of seconds this request is able to be kept open without the client
+     * closing it. This depends on the vendor.
+     *
+     * @access public
+     * @return boolean
+     */
+    static public function GetExpectedConnectionTimeout() {
+        // Different vendors implement different connection timeouts.
+        // In order to optimize processing, we return a specific time for the major
+        // classes currently known (feedback welcome).
+        // The amount of time returned is somehow lower than the max timeout so we have
+        // time for processing.
+
+        // Apple and Windows Phone have higher timeouts (4min = 240sec)
+        if (in_array(self::GetDeviceType(), array("iPod", "iPad", "iPhone", "WP"))) {
+            return 200;
+        }
+        // Samsung devices have a intermediate timeout (90sec)
+        if (in_array(self::GetDeviceType(), array("SAMSUNGGTI"))) {
+            return 50;
+        }
+
+        // for all other devices, a timeout of 30 seconds is expected
+        return 20;
+    }
 
     /**----------------------------------------------------------------------------------------------------------
      * Private stuff
