@@ -61,6 +61,7 @@ class Sync extends RequestProcessor {
         $status = SYNC_STATUS_SUCCESS;
         $wbxmlproblem = false;
         $emptysync = false;
+        $globallyExportedItems = 0;
 
 
         // check if the hierarchySync was fully completed
@@ -231,8 +232,8 @@ class Sync extends RequestProcessor {
                     if(self::$decoder->getElementStartTag(SYNC_WINDOWSIZE)) {
                         $ws = self::$decoder->getElementContent();
                         // normalize windowsize - see ZP-477
-                        if ($ws == 0 || $ws > 512)
-                            $ws = 512;
+                        if ($ws == 0 || $ws > WINDOW_SIZE_MAX)
+                            $ws = WINDOW_SIZE_MAX;
 
                         $spa->SetWindowSize($ws);
 
@@ -426,7 +427,7 @@ class Sync extends RequestProcessor {
                                 $clientid = false;
 
                             // Get the SyncMessage if sent
-                            if(self::$decoder->getElementStartTag(SYNC_DATA)) {
+                            if(($el = self::$decoder->getElementStartTag(SYNC_DATA)) && ($el[EN_FLAGS] & EN_FLAGS_CONTENT)) {
                                 $message = ZPush::getSyncObjectFromFolderClass($spa->GetContentClass());
                                 $message->Decode(self::$decoder);
 
@@ -522,6 +523,7 @@ class Sync extends RequestProcessor {
 
             if (self::$decoder->getElementStartTag(SYNC_WINDOWSIZE)) {
                 $sc->SetGlobalWindowSize(self::$decoder->getElementContent());
+                ZLog::Write(LOGLEVEL_DEBUG, "Sync(): Global WindowSize requested: ". $sc->GetGlobalWindowSize());
                 if(!self::$decoder->getElementEndTag()) // SYNC_WINDOWSIZE
                     return false;
             }
@@ -567,15 +569,13 @@ class Sync extends RequestProcessor {
 
             // update a few values
             foreach($sc as $folderid => $spa) {
-                // manually set getchanges parameter for this collection
-                $sc->AddParameter($spa, "getchanges", true);
+                // manually set getchanges parameter for this collection if it is synchronized
+                if ($spa->HasSyncKey()) {
+                    $sc->AddParameter($spa, "getchanges", true);
 
-                // set new global windowsize without marking the SPA as changed
-                if ($sc->GetGlobalWindowSize())
-                    $spa->SetWindowSize($sc->GetGlobalWindowSize(), false);
-
-                // announce WindowSize to DeviceManager
-                self::$deviceManager->SetWindowSize($folderid, $spa->GetWindowSize());
+                    // announce WindowSize to DeviceManager
+                    self::$deviceManager->SetWindowSize($folderid, $spa->GetWindowSize());
+                }
             }
             if (!$sc->HasCollections())
                 $status = SYNC_STATUS_SYNCREQUESTINCOMPLETE;
@@ -623,12 +623,25 @@ class Sync extends RequestProcessor {
                     }
                 }
 
-                // in case there are no changes, we can reply with an empty response
-                if (!$foundchanges && $status == SYNC_STATUS_SUCCESS){
-                    ZLog::Write(LOGLEVEL_DEBUG, "No changes found. Replying with empty response and closing connection.");
-                    self::$specialHeaders = array();
-                    self::$specialHeaders[] = "Connection: close";
-                    return true;
+                // in case there are no changes and no other request has synchronized while we waited, we can reply with an empty response
+                if (!$foundchanges && $status == SYNC_STATUS_SUCCESS) {
+                    // if there were changes to the SPA or CPOs we need to save this before we terminate
+                    // only save if the state was not modified by some other request, if so, return state invalid status
+                    foreach($sc as $folderid => $spa) {
+                        if (self::$deviceManager->CheckHearbeatStateIntegrity($spa->GetFolderId(), $spa->GetUuid(), $spa->GetUuidCounter())) {
+                            $status = SYNC_COMMONSTATUS_SYNCSTATEVERSIONINVALID;
+                        }
+                        else {
+                            $sc->SaveCollection($spa);
+                        }
+                    }
+
+                    if ($status == SYNC_STATUS_SUCCESS) {
+                        ZLog::Write(LOGLEVEL_DEBUG, "No changes found and no other process changed states. Replying with empty response and closing connection.");
+                        self::$specialHeaders = array();
+                        self::$specialHeaders[] = "Connection: close";
+                        return true;
+                    }
                 }
 
                 if ($foundchanges) {
@@ -739,12 +752,26 @@ class Sync extends RequestProcessor {
                             continue;
                         }
 
-                        // Get a new sync key to output to the client if any changes have been send or will are available
+                        // Get a new sync key to output to the client if any changes have been send by the mobile or a new synckey is to be sent
                         if (!empty($actiondata["modifyids"]) ||
                             !empty($actiondata["clientids"]) ||
                             !empty($actiondata["removeids"]) ||
-                            $changecount > 0 || (! $spa->HasSyncKey() && $status == SYNC_STATUS_SUCCESS))
+                            (! $spa->HasSyncKey() && $status == SYNC_STATUS_SUCCESS)) {
                                 $spa->SetNewSyncKey(self::$deviceManager->GetStateManager()->GetNewSyncKey($spa->GetSyncKey()));
+                        }
+                        // get a new synckey only if we did not reach the global limit yet
+                        else {
+                            // when reaching the global limit for changes of all collections, stop processing other collections (ZP-697)
+                            if ($sc->GetGlobalWindowSize() <= $globallyExportedItems) {
+                                ZLog::Write(LOGLEVEL_DEBUG, "Global WindowSize for amount of exported changes reached, omitting output for collection.");
+                                continue;
+                            }
+
+                            // get a new synckey if there are changes are we did not reach the limit yet
+                            if ($changecount > 0) {
+                                $spa->SetNewSyncKey(self::$deviceManager->GetStateManager()->GetNewSyncKey($spa->GetSyncKey()));
+                            }
+                        }
 
                         self::$encoder->startTag(SYNC_FOLDER);
 
@@ -879,6 +906,13 @@ class Sync extends RequestProcessor {
                         if($sc->GetParameter($spa, "getchanges") && $spa->HasFolderId() && $spa->HasContentClass() && $spa->HasSyncKey()) {
                             $windowSize = self::$deviceManager->GetWindowSize($spa->GetFolderId(), $spa->GetContentClass(), $spa->GetUuid(), $spa->GetUuidCounter(), $changecount);
 
+                            // limit windowSize to the max available limit of the global window size left
+                            $globallyAvailable = $sc->GetGlobalWindowSize() - $globallyExportedItems;
+                            if ($changecount > $globallyAvailable && $windowSize > $globallyAvailable) {
+                                ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): Limit window size to %d as the global window size limit will be reached", $globallyAvailable));
+                                $windowSize = $globallyAvailable;
+                            }
+                            // send <MoreAvailable/> if there are more changes than fit in the folder windowsize
                             if($changecount > $windowSize) {
                                 self::$encoder->startTag(SYNC_MOREAVAILABLE, false, true);
                             }
@@ -898,6 +932,8 @@ class Sync extends RequestProcessor {
                                     if(!is_array($progress))
                                         break;
                                     $n++;
+                                    if ($n % 10 == 0)
+                                        self::$topCollector->AnnounceInformation(sprintf("Streamed data of %d objects out of %d", $n, (($changecount > $windowSize)?$windowSize:$changecount)));
                                 }
                                 catch (SyncObjectBrokenException $mbe) {
                                     $brokenSO = $mbe->GetSyncObject();
@@ -912,6 +948,15 @@ class Sync extends RequestProcessor {
                                         self::$deviceManager->AnnounceIgnoredMessage($spa->GetFolderId(), $brokenSO->id, $brokenSO);
                                     }
                                 }
+                                // something really bad happened while exporting changes
+                                catch (StatusException $stex) {
+                                    $status = $stex->getCode();
+                                    // during export we found out that the states should be thrown away (ZP-623)
+                                    if ($status == SYNC_STATUS_INVALIDSYNCKEY) {
+                                        self::$deviceManager->ForceFolderResync($spa->GetFolderId());
+                                        break;
+                                    }
+                                }
 
                                 if($n >= $windowSize) {
                                     ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): Exported maxItems of messages: %d / %d", $n, $changecount));
@@ -927,9 +972,12 @@ class Sync extends RequestProcessor {
 
                             self::$encoder->endTag();
                             self::$topCollector->AnnounceInformation(sprintf("Outgoing %d objects%s", $n, ($n >= $windowSize)?" of ".$changecount:""), true);
+                            $globallyExportedItems += $n;
 
-                            // update folder status
-                            $spa->SetFolderSyncRemaining($changecount);
+                            // update folder status, if there is something set
+                            if ($spa->GetFolderSyncRemaining() && $changecount > 0) {
+                                $spa->SetFolderSyncRemaining($changecount);
+                            }
                             // changecount is initialized with 'false', so 0 means no changes!
                             if ($changecount === 0 || ($changecount !== false && $changecount <= $windowSize))
                                 self::$deviceManager->SetFolderSyncStatus($folderid, DeviceManager::FLD_SYNC_COMPLETED);
@@ -966,13 +1014,14 @@ class Sync extends RequestProcessor {
                                 ZLog::Write(LOGLEVEL_ERROR, sprintf("HandleSync(): error saving '%s' - no state information available", $spa->GetNewSyncKey()));
                         }
 
-                        // reset status for the next folder
-                        $status = SYNC_STATUS_SUCCESS;
-
                         // save SyncParameters
                         if ($status == SYNC_STATUS_SUCCESS && empty($actiondata["fetchids"]))
                             $sc->SaveCollection($spa);
 
+                        // reset status for the next folder
+                        $status = SYNC_STATUS_SUCCESS;
+
+
                     } // END foreach collection
                 }
                 self::$encoder->endTag(); //SYNC_FOLDERS
