@@ -917,6 +917,22 @@ class Utils {
 
         return 0;
     }
+
+    /**
+     * Returns the local part from email address.
+     *
+     * @param string $email
+     *
+     * @access public
+     * @return string
+     */
+    public static function GetLocalPartFromEmail($email) {
+        $pos = strpos($email, '@');
+        if ($pos === false) {
+            return $email;
+        }
+        return substr($email, 0, $pos);
+    }
 }
 
 
