@@ -9,7 +9,7 @@
 *
 * Created   :   14.05.2010
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2015 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -92,7 +92,8 @@ include('lib/syncobjects/syncuserinformation.php');
 include('lib/syncobjects/syncdeviceinformation.php');
 include('lib/syncobjects/syncdevicepassword.php');
 include('lib/syncobjects/syncitemoperationsattachment.php');
-include('config.php');
+if (!defined('ZPUSH_CONFIG')) define('ZPUSH_CONFIG', 'config.php');
+include_once(ZPUSH_CONFIG);
 include('version.php');
 
 /**
@@ -112,7 +113,7 @@ include('version.php');
         if (! ZPushAdminCLI::SureWhatToDo()) {
             // show error message if available
             if (ZPushAdminCLI::GetErrorMessage())
-                echo "ERROR: ". ZPushAdminCLI::GetErrorMessage() . "\n";
+                fwrite(STDERR, ZPushAdminCLI::GetErrorMessage() . "\n");
 
             echo ZPushAdminCLI::UsageInstructions();
             exit(1);
@@ -139,6 +140,14 @@ class ZPushAdminCLI {
     const COMMAND_SHOWLASTSYNC = 8;
     const COMMAND_RESYNCFOLDER = 9;
     const COMMAND_FIXSTATES = 10;
+    const COMMAND_RESYNCHIERARCHY = 11;
+
+    const TYPE_OPTION_EMAIL = "email";
+    const TYPE_OPTION_CALENDAR = "calendar";
+    const TYPE_OPTION_CONTACT = "contact";
+    const TYPE_OPTION_TASK = "task";
+    const TYPE_OPTION_NOTE = "note";
+    const TYPE_OPTION_HIERARCHY = "hierarchy";
 
     static private $command;
     static private $user = false;
@@ -153,27 +162,29 @@ class ZPushAdminCLI {
      * @access public
      */
     static public function UsageInstructions() {
+        $types = "'".self::TYPE_OPTION_EMAIL."', '".self::TYPE_OPTION_CALENDAR."', '".self::TYPE_OPTION_CONTACT."', '".self::TYPE_OPTION_TASK."' or '".self::TYPE_OPTION_NOTE."'";
         return  "Usage:\n\tz-push-admin.php -a ACTION [options]\n\n" .
                 "Parameters:\n\t-a list/wipe/remove/resync/clearloop\n\t[-u] username\n\t[-d] deviceid\n\n" .
                 "Actions:\n" .
-                "\tlist\t\t\t\t Lists all devices and synchronized users\n" .
-                "\tlist -u USER\t\t\t Lists all devices of user USER\n" .
-                "\tlist -d DEVICE\t\t\t Lists all users of device DEVICE\n" .
-                "\tlastsync\t\t\t Lists all devices and synchronized users and the last synchronization time\n" .
-                "\twipe -u USER\t\t\t Remote wipes all devices of user USER\n" .
-                "\twipe -d DEVICE\t\t\t Remote wipes device DEVICE\n" .
-                "\twipe -u USER -d DEVICE\t\t Remote wipes device DEVICE of user USER\n" .
-                "\tremove -u USER\t\t\t Removes all state data of all devices of user USER\n" .
-                "\tremove -d DEVICE\t\t Removes all state data of all users synchronized on device DEVICE\n" .
-                "\tremove -u USER -d DEVICE\t Removes all related state data of device DEVICE of user USER\n" .
-                "\tresync -u USER -d DEVICE\t Resynchronizes all data of device DEVICE of user USER\n" .
-                "\tresync -t TYPE \t\t\t Resynchronizes all folders of type 'email', 'calendar', 'contact', 'task' or 'note' for all devices and users.\n" .
-                "\tresync -t TYPE -u USER \t\t Resynchronizes all folders of type 'email', 'calendar', 'contact', 'task' or 'note' for the user USER.\n" .
-                "\tresync -t TYPE -u USER -d DEVICE Resynchronizes all folders of type 'email', 'calendar', 'contact', 'task' or 'note' for a specified device and user.\n" .
-                "\tresync -t FOLDERID -u USER\t Resynchronize the specified folder id only. The USER should be specified.\n" .
-                "\tclearloop\t\t\t Clears system wide loop detection data\n" .
-                "\tclearloop -d DEVICE -u USER\t Clears all loop detection data of a device DEVICE and an optional user USER\n" .
-                "\tfixstates\t\t\t Checks the states for integrity and fixes potential issues\n" .
+                "\tlist\t\t\t\t\t Lists all devices and synchronized users\n" .
+                "\tlist -u USER\t\t\t\t Lists all devices of user USER\n" .
+                "\tlist -d DEVICE\t\t\t\t Lists all users of device DEVICE\n" .
+                "\tlastsync\t\t\t\t Lists all devices and synchronized users and the last synchronization time\n" .
+                "\twipe -u USER\t\t\t\t Remote wipes all devices of user USER\n" .
+                "\twipe -d DEVICE\t\t\t\t Remote wipes device DEVICE\n" .
+                "\twipe -u USER -d DEVICE\t\t\t Remote wipes device DEVICE of user USER\n" .
+                "\tremove -u USER\t\t\t\t Removes all state data of all devices of user USER\n" .
+                "\tremove -d DEVICE\t\t\t Removes all state data of all users synchronized on device DEVICE\n" .
+                "\tremove -u USER -d DEVICE\t\t Removes all related state data of device DEVICE of user USER\n" .
+                "\tresync -u USER -d DEVICE\t\t Resynchronizes all data of device DEVICE of user USER\n" .
+                "\tresync -t TYPE \t\t\t\t Resynchronizes all folders of type $types for all devices and users.\n" .
+                "\tresync -t TYPE -u USER \t\t\t Resynchronizes all folders of type $types for the user USER.\n" .
+                "\tresync -t TYPE -u USER -d DEVICE\t Resynchronizes all folders of type $types for a specified device and user.\n" .
+                "\tresync -t FOLDERID -u USER\t\t Resynchronize the specified folder id only. The USER should be specified for better performance.\n" .
+                "\tresync -t hierarchy -u USER -d DEVICE\t Resynchronize the folder hierarchy data for an optional USER and optional DEVICE.\n" .
+                "\tclearloop\t\t\t\t Clears system wide loop detection data\n" .
+                "\tclearloop -d DEVICE -u USER\t\t Clears all loop detection data of a device DEVICE and an optional user USER\n" .
+                "\tfixstates\t\t\t\t Checks the states for integrity and fixes potential issues\n" .
                 "\n";
     }
 
@@ -184,8 +195,8 @@ class ZPushAdminCLI {
      * @access public
      */
     static public function CheckEnv() {
-        if (!isset($_SERVER["TERM"]) || !isset($_SERVER["LOGNAME"]))
-            self::$errormessage = "This script should not be called in a browser.";
+        if (php_sapi_name() != "cli")
+            self::$errormessage = "This script can only be called from the CLI.";
 
         if (!function_exists("getopt"))
             self::$errormessage = "PHP Function getopt not found. Please check your PHP version and settings.";
@@ -228,6 +239,22 @@ class ZPushAdminCLI {
         elseif (isset($options['type']) && !empty($options['type']))
             self::$type = strtolower(trim($options['type']));
 
+        // if type is set, it must be one of known types or a 44 byte long folder id
+        if (self::$type !== false) {
+            if (self::$type !== self::TYPE_OPTION_EMAIL &&
+                self::$type !== self::TYPE_OPTION_CALENDAR &&
+                self::$type !== self::TYPE_OPTION_CONTACT &&
+                self::$type !== self::TYPE_OPTION_TASK &&
+                self::$type !== self::TYPE_OPTION_NOTE &&
+                self::$type !== self::TYPE_OPTION_HIERARCHY &&
+                strlen(self::$type) !== 44) {
+                    self::$errormessage = "Wrong 'type'. Possible values are: ".
+                        "'".self::TYPE_OPTION_EMAIL."', '".self::TYPE_OPTION_CALENDAR."', '".self::TYPE_OPTION_CONTACT."', '".self::TYPE_OPTION_TASK."', '".self::TYPE_OPTION_NOTE."', ".self::TYPE_OPTION_HIERARCHY."' ".
+                        "or a 44 byte long folder id (as hex).";
+                    return;
+                }
+        }
+
         // get a command for the requested action
         switch ($action) {
             // list data
@@ -277,6 +304,9 @@ class ZPushAdminCLI {
                     else
                         self::$command = self::COMMAND_RESYNCDEVICE;
                 }
+                else if (self::$type === self::TYPE_OPTION_HIERARCHY) {
+                    self::$command = self::COMMAND_RESYNCHIERARCHY;
+                }
                 else {
                     self::$command = self::COMMAND_RESYNCFOLDER;
                 }
@@ -387,6 +417,18 @@ class ZPushAdminCLI {
                     self::CommandResyncFolder();
                     break;
 
+                case self::COMMAND_RESYNCHIERARCHY:
+                    if (self::$device == false && self::$user == false) {
+                        echo "Are you sure you want to re-synchronize the hierarchy of all devices and users [y/N]: ";
+                        $confirm  =  strtolower(trim(fgets(STDIN)));
+                        if ( !($confirm === 'y' || $confirm === 'yes')) {
+                            echo "Aborted!\n";
+                            exit(1);
+                        }
+                    }
+                    self::CommandResyncHierarchy();
+                    break;
+
             case self::COMMAND_CLEARLOOP:
                 self::CommandClearLoopDetectionData();
                 break;
@@ -465,10 +507,18 @@ class ZPushAdminCLI {
     static public function CommandDeviceUsers() {
         $users = ZPushAdmin::ListUsers(self::$device);
 
-        if (empty($users))
+        if (empty($users)) {
             echo "\tno user data synchronized to device\n";
+        }
+        // if a user is specified, we only want to see the devices of this one
+        else if (self::$user !== false && !in_array(self::$user, $users)) {
+            printf("\tuser '%s' not known in device data '%s'\n", self::$user, self::$device);
+        }
 
         foreach ($users as $user) {
+            if (self::$user !== false && strtolower($user) !== self::$user) {
+                continue;
+            }
             echo "Synchronized by user: ". $user. "\n";
             self::printDeviceData(self::$device, $user);
         }
@@ -557,6 +607,36 @@ class ZPushAdminCLI {
 
     }
 
+    /**
+     * Command "Resync hierarchy"
+     * Resyncs a folder type of a specific device/user or of all users
+     *
+     * @return
+     * @access public
+     */
+    static public function CommandResyncHierarchy() {
+        // if no device is specified, search for all devices of a user. If user is not set, all devices are returned.
+        if (self::$device === false) {
+            $devicelist = ZPushAdmin::ListDevices(self::$user);
+            if (empty($devicelist)) {
+                echo "\tno devices/users found\n";
+                return true;
+            }
+        }
+        else
+            $devicelist = array(self::$device);
+
+        foreach ($devicelist as $deviceId) {
+            $users = ZPushAdmin::ListUsers($deviceId);
+            foreach ($users as $user) {
+                if (self::$user && self::$user != $user)
+                    continue;
+                self::resyncHierarchy($deviceId, $user);
+            }
+        }
+
+    }
+
     /**
      * Command to clear the loop detection data
      * Mobiles may enter loop detection (one-by-one synchring due to timeouts / erros).
@@ -639,6 +719,20 @@ class ZPushAdminCLI {
         echo sprintf("Resync of %d folders of type %s on device '%s' of user '%s': %s\n", count($folders), $type, $deviceId, $user, ($stat)?'Requested':ZLog::GetLastMessage(LOGLEVEL_ERROR));
     }
 
+    /**
+     * Resynchronizes the hierarchy of a device & user
+     *
+     * @param string    $deviceId       the id of the device
+     * @param string    $user           the user
+     *
+     * @return
+     * @access private
+     */
+    static private function resyncHierarchy($deviceId, $user) {
+        $stat = ZPushAdmin::ResyncHierarchy($user, $deviceId);
+        echo sprintf("Removing hierarchy information for resync on device '%s' of user '%s': %s\n",  $deviceId, $user, ($stat)?'Requested':ZLog::GetLastMessage(LOGLEVEL_ERROR));
+    }
+
     /**
      * Fixes the states for potential issues
      *
